'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var reactTreat = require('react-treat');
var treat = require('treat');

const GlazeContext = React__default.createContext(undefined);
function ThemeProvider({
  theme,
  children
}) {
  return React__default.createElement(reactTreat.TreatProvider, {
    theme: theme.ref
  }, React__default.createElement(GlazeContext.Provider, {
    value: theme
  }, children));
}
function useTheme() {
  const theme = React.useContext(GlazeContext);
  if (!theme) throw new Error('No glaze theme provided');
  return theme;
}

// eslint-disable-next-line import/prefer-default-export
function modularScale(ratio, steps = [-1, -0.5, 0, +1, +2, +3, +4, +5, +6]) {
  const result = {};
  steps.forEach(step => {
    // Rounding to 3 decimal places
    result[step] = `${Math.round(ratio ** step * 1e3) / 1e3}rem`;
  });
  return result;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function createTheme(tokens, localDebugName) {
  const {
    breakpoints,
    shorthands,
    aliases
  } = tokens;
  return {
    ref: treat.createTheme(tokens, localDebugName),
    breakpoints,
    shorthands,
    aliases
  };
} // TODO: symmetricScale()

const spacing = {
  0: 0,
  '1px': 1,
  1: '.25rem',
  2: '.5rem',
  3: '.75rem',
  4: '1rem',
  5: '1.25rem',
  6: '1.5rem',
  8: '2rem',
  10: '2.5rem',
  12: '3rem',
  16: '4rem',
  20: '5rem',
  24: '6rem',
  32: '8rem',
  40: '10rem',
  48: '12rem',
  56: '14rem',
  64: '16rem'
};
const defaultTheme = {
  breakpoints: [640, 768, 1024, 1280],
  scales: {
    spacing,
    size: _objectSpread2({}, spacing, {
      auto: 'auto',
      '100%': '100%'
    }),
    fontSize: modularScale(1.333),
    lineHeight: {
      1: 1,
      tight: 1.25,
      snug: 1.375,
      base: 1.5,
      relaxed: 1.625,
      loose: 2
    },
    letterSpacing: {
      wide: '.025em'
    },
    borderWidth: {
      1: 1,
      2: 2,
      4: 4,
      8: 8
    },
    radius: {
      sm: '.125rem',
      md: '.25rem',
      lg: '.5rem',
      full: 9999
    },
    shadow: {
      sm: '0 1px 3px 0 rgba(0, 0, 0, 0.1), 0 1px 2px 0 rgba(0, 0, 0, 0.06)',
      md: '0 4px 6px -1px rgba(0, 0, 0, 0.1), 0 2px 4px -1px rgba(0, 0, 0, 0.06)',
      lg: '0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05)',
      xl: '0 20px 25px -5px rgba(0, 0, 0, 0.1), 0 10px 10px -5px rgba(0, 0, 0, 0.04)',
      inner: 'inset 0 2px 4px 0 rgba(0, 0, 0, 0.06)',
      outline: '0 0 0 3px rgba(66, 153, 225, 0.5)'
    }
  },
  shorthands: {
    // TODO: Remove if widely supported by browsers
    inset: ['top', 'right', 'bottom', 'left'],
    insetX: ['left', 'right'],
    insetY: ['top', 'bottom'],
    // TODO: Remove if widely supported by browsers
    size: ['width', 'height'],
    paddingX: ['paddingLeft', 'paddingRight'],
    paddingY: ['paddingTop', 'paddingBottom'],
    marginX: ['marginLeft', 'marginRight'],
    marginY: ['marginTop', 'marginBottom']
  },
  aliases: {
    p: 'padding',
    px: 'paddingX',
    py: 'paddingY',
    pt: 'paddingTop',
    pr: 'paddingRight',
    pb: 'paddingBottom',
    pl: 'paddingLeft',
    // TODO: For logical properties, e.g. lpi, lpis, lpie, lpb, lpbs, lpbe
    m: 'margin',
    mx: 'marginX',
    my: 'marginY',
    mt: 'marginTop',
    mr: 'marginRight',
    mb: 'marginBottom',
    ml: 'marginLeft',
    // TODO: For logical properties, e.g. lmi, lmis, lmie, lmb, lmbs, lmbe
    bg: 'background'
  },
  resolvers: {
    top: 'spacing',
    right: 'spacing',
    bottom: 'spacing',
    left: 'spacing',
    zIndex: 'zIndex',
    flexBasis: 'size',
    gridGap: 'spacing',
    gridRowGap: 'spacing',
    gridColumnGap: 'spacing',
    gap: 'spacing',
    rowGap: 'spacing',
    columnGap: 'spacing',
    width: 'size',
    minWidth: 'size',
    maxWidth: 'size',
    height: 'size',
    minHeight: 'size',
    maxHeight: 'size',
    padding: 'spacing',
    // TODO: paddingInline: 'spacing',
    // TODO: paddingInlineStart: 'spacing',
    // TODO: paddingInlineEnd: 'spacing',
    // TODO: paddingBlock: 'spacing',
    // TODO: paddingBlockStart: 'spacing',
    // TODO: paddingBlockEnd: 'spacing',
    paddingTop: 'spacing',
    paddingRight: 'spacing',
    paddingBottom: 'spacing',
    paddingLeft: 'spacing',
    margin: 'spacing',
    // TODO: marginInline: 'spacing',
    // TODO: marginInlineStart: 'spacing',
    // TODO: marginInlineEnd: 'spacing',
    // TODO: marginBlock: 'spacing',
    // TODO: marginBlockStart: 'spacing',
    // TODO: marginBlockEnd: 'spacing',
    marginTop: 'spacing',
    marginRight: 'spacing',
    marginBottom: 'spacing',
    marginLeft: 'spacing',
    fontFamily: 'fontFamily',
    fontSize: 'fontSize',
    fontWeight: 'fontWeight',
    lineHeight: 'lineHeight',
    color: 'color',
    textShadow: 'shadow',
    letterSpacing: 'letterSpacing',
    background: 'color',
    backgroundColor: 'color',
    border: 'border',
    borderTop: 'border',
    borderRight: 'border',
    borderBottom: 'border',
    borderLeft: 'border',
    borderColor: 'color',
    borderTopColor: 'color',
    borderRightColor: 'color',
    borderBottomColor: 'color',
    borderLeftColor: 'color',
    borderWidth: 'borderWidth',
    borderTopWidth: 'borderWidth',
    borderRightWidth: 'borderWidth',
    borderBottomWidth: 'borderWidth',
    borderLeftWidth: 'borderWidth',
    borderRadius: 'radius',
    borderTopLeftRadius: 'radius',
    borderTopRightRadius: 'radius',
    borderBottomRightRadius: 'radius',
    borderBottomLeftRadius: 'radius',
    outlineColor: 'color',
    boxShadow: 'shadow',
    opacity: 'opacity'
  }
};

var styleRefs = treat.styleMap(theme => {
  const result = {};
  Object.entries(theme.resolvers).forEach(([property, scale]) => {
    if (scale) {
      const tokens = Object.entries(theme.scales[scale] || {});
      tokens.forEach(([key, value]) => {
        result[`${property}.${key}`] = {
          [property]: value
        };
      });
    }
  });
  return result;
});

function useStyling() {
  const staticClassNames = reactTreat.useStyles(styleRefs);
  const theme = useTheme();
  return function sx(themedStyle) {
    let className = '';
    const style = {}; // Prefer performance over clarity for the runtime
    // eslint-disable-next-line guard-for-in, no-restricted-syntax

    for (const alias in themedStyle) {
      const value = themedStyle[alias];
      const shorthand = theme.aliases[alias] || alias; // eslint-disable-next-line no-loop-func

      (theme.shorthands[shorthand] || [shorthand]).forEach(key => {
        // TODO: Support selectors and media queries
        if (typeof value !== 'object') {
          const staticClassName = staticClassNames[`${key}.${value}`];

          if (staticClassName) {
            className += `${staticClassName} `;
          } else {
            style[key] = value;
          }
        }
      });
    }

    return {
      className,
      style
    };
  };
}

exports.ThemeProvider = ThemeProvider;
exports.createTheme = createTheme;
exports.defaultTheme = defaultTheme;
exports.modularScale = modularScale;
exports.useStyling = useStyling;
exports.useTheme = useTheme;
//# sourceMappingURL=index.js.map
